/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | www.openfoam.com
     \\/     M anipulation  |
-------------------------------------------------------------------------------
    Copyright (C) 2014-2016 OpenFOAM Foundation
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "algebraic.H"
#include "addToRunTimeSelectionTable.H"
#include "twoPhaseMixtureThermo.H"

// * * * * * * * * * * * * * * Static Data Members * * * * * * * * * * * * * //

namespace Foam
{
namespace relativeVelocityModels
{
    defineTypeNameAndDebug(algebraic, 0);
    addToRunTimeSelectionTable(relativeVelocityModel, algebraic, components);
}
}


// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::relativeVelocityModels::algebraic::algebraic
(
    const twoPhaseMixtureThermo& mixture,
    const fvMesh& mesh
)
:
    relativeVelocityModel(mixture, mesh),
    residualRe_("residualRe", dimless, optionalSubDict(type() + "Coeffs")),
    dd_("dd", dimLength, optionalSubDict(type() + "Coeffs")),
    turbulenceCorrect_(optionalSubDict(type() + "Coeffs").getOrDefault("turbulenceCorrect", false)),
    rho0_("rho0",dimDensity,1)
{
    correct();
}


// * * * * * * * * * * * * * * * * Destructor  * * * * * * * * * * * * * * * //

Foam::relativeVelocityModels::algebraic::~algebraic()
{}


// * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * * //

void Foam::relativeVelocityModels::algebraic::correct()
{
    volScalarField betac(alphac_*rhoc_);
    volScalarField betad(alphad_*rhod_);

    /*************** calculate Re  ************/
    volScalarField muc_ = const_cast<twoPhaseMixtureThermo&>(mixture_).thermo2().mu();
    Info<< "mu max/min : " << min(muc_).value() << "/"  << max(muc_).value() << endl;

    Udc_ = mesh_.lookupObject<volVectorField>("Udc");
    Info << "max(mag(Udc)): " << max(mag(Udc_)).value() << endl;

    volScalarField Re_p(rhoc_ * dd_ * mag(Udc_) / muc_ + residualRe_);
    Info<< "Re max/min : " << min(Re_p).value() << "/"  << max(Re_p).value() << endl;

    /*************** calculate drag force  ************/
    volScalarField fd = 1. + 0.15 * pow(Re_p, 0.687);
    forAll(Re_p, i)
    {
        if (Re_p[i] > 1000)
        {
            fd[i] = 0.018333 * Re_p[i];
        }
    }
    Info<< "fd max/min : " << min(fd).value() << "/"  << max(fd).value() << endl;

    // const meshObjects::gravity& g = meshObjects::gravity::New(mesh_.lookupObject<volVectorField>("U").db().time());
    /*
    meshObjects::gravity g
    (
        mixture_.U().mesh().lookupObject<meshObjects::gravity>("g")
    );
    */
   /*************** calculate the acceleration ************/
    volVectorField a = mesh_.lookupObject<volVectorField>("AA");//g - U*fvc::div(U) ;//- fvc::ddt(U); //
    Info << "max(mag(a)): " << max(mag(a)).value() << endl;

    /*************** calculate relative velocity ************/
    Udc_ = (rhod_ - rho())*sqr(dd_)/18./muc_/fd*a;

    forAll(Udc_, celli)
    {
        const scalar magAA = mag(Udc_[celli]);
        if (magAA > 50)
            Udc_[celli] = (Udc_[celli] / magAA) * 50; // 或使用 normalise(AA[celli]) * 2000
    }

    forAll(Udc_.boundaryField(), patchi)
    {
        auto& bField = Udc_.boundaryFieldRef()[patchi];
        forAll(bField, facei)
        {
            const scalar magAA = mag(bField[facei]);
            if (magAA > 50)
                bField[facei] = (bField[facei] / magAA) * 50;
        }
    }

    Info << "max(mag(Udc)): " << max(mag(Udc_)).value() << endl;  

    Udm_ =   betac / rho() * Udc_;
    Ucm_ = - betad / rho() * Udc_;    
}


// ************************************************************************* //
